#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
from pathlib import Path
import uuid
from datetime import datetime
import subprocess
import shlex

from poppy.core.conf import settings
from poppy.core.db.connector import Connector
from poppy.core.generic.requests import download_file
from poppy.core.logger import logger
from poppy.core.task import Task

__all__ = ["LoadIdbDump"]

from roc.idb.constants import (
    MIB_RELEASE_URL_PATTERN,
    IDB_INSTALL_DIR,
    TIMEOUT,
    POSTGRES_DB_PORT,
)


class LoadIdbDump(Task):
    plugin_name = "roc.idb"
    name = "idb_load_idb_dump"

    @Connector.if_connected(settings.PIPELINE_DATABASE)
    def setup_inputs(self):
        # Get input arguments ===
        self.dump_file = self.pipeline.get("dump_file", default=[None])[0]
        self.install_dir = self.pipeline.get("install_dir", default=[IDB_INSTALL_DIR])[
            0
        ]
        self.auth = tuple(self.pipeline.get("auth", default=(None, None)))
        self.timeout = self.pipeline.get("timeout", default=[TIMEOUT])
        self.pg_restore = self.pipeline.get("pg_restore", default=False)
        self.clear_temp = self.pipeline.get("clear_temp", default=False)
        self.force = self.pipeline.get("force", default=False)
        self.shell = self.pipeline.get("shell", default=False)

        # Building a valid dump file path before ingestion
        if self.dump_file is None and self.mib_version:
            # If no dump file provided, then try to retrieve dump file
            # from ROC gitlab server using mib version

            # First build the URL with input MIB version
            self.dump_file = MIB_RELEASE_URL_PATTERN.format(
                mib_version=self.mib_version
            )
        elif self.dump_file is None and self.mib_version is None:
            raise ValueError(
                "Not enough information to continue: \n "
                "at least one of the following inputs must be passed: "
                "dump_file or mib_version "
            )

        # get the database information
        db_info = self.pipeline.properties.configuration["pipeline.databases"][0]
        self.db_user, self.db_pass = db_info["login_info"]["admin"].split(":")
        self.db_host = db_info["login_info"]["address"]
        self.db_name = db_info["login_info"]["database"]
        try:
            self.db_port = db_info["login_info"]["port"]
        except KeyError:
            self.db_port = POSTGRES_DB_PORT

    def run(self):
        # Define task job ID (long and short)
        self.job_uuid = str(uuid.uuid4())
        self.job_id = self.job_uuid[:8]
        logger.info(f"Task {self.job_id} is starting")
        try:
            self.setup_inputs()
        except Exception as e:
            logger.error(f"Initializing inputs has failed for task {self.job_id}!")
            logger.debug(e)
            self.pipeline.exit()
            return

        # Get idb dump filename
        dump_basename = os.path.basename(self.dump_file)

        # Check if input dump file path is a URL or a local path
        if self.dump_file.startswith("https"):
            # if it is a URL, then download first the file

            # Build local file path
            Path(self.install_dir).mkdir(parents=True, exist_ok=True)
            local_filepath = os.path.join(self.install_dir, dump_basename)

            # Call the gitlab API using the user token
            if not os.path.isfile(local_filepath) or self.force:
                try:
                    logger.info(
                        f"Downloading {self.dump_file} in {local_filepath}, please wait"
                    )
                    download_file(
                        local_filepath,
                        self.dump_file,
                        auth=self.auth,
                    )
                except Exception as e:
                    logger.error(f"Downloading {self.dump_file} has failed!")
                    logger.debug(e)
                else:
                    logger.info(f"{local_filepath} downloaded")

        else:
            logger.info(f"Use {self.dump_file} as IDB dump file")
            local_filepath = self.dump_file

        # Now run psql tool to insert IDB dump file content
        # into the ROC database
        if self.pg_restore:
            exe = "pg_restore"
            sep = ""
        else:
            exe = "psql"
            sep = ""

        cmd = " ".join(
            [
                exe,
                f"-h {self.db_host}",
                f"-U {self.db_user}",
                f"-d {self.db_name}",
                f"-p {self.db_port}",
                sep,
                f"-f {local_filepath}",
            ]
        )

        process = None
        if os.path.isfile(local_filepath):
            try:
                run_start_time = datetime.now()
                logger.info(f'Running "{cmd}" started on {run_start_time}')

                # Let the shell out of this (i.e. shell=False)
                process = subprocess.run(
                    shlex.split(cmd),
                    shell=self.shell,
                    check=True,
                    stdin=subprocess.PIPE,
                    stdout=subprocess.PIPE,
                    env={"PGPASSWORD": self.db_pass},
                )
            except subprocess.CalledProcessError as e:
                logger.exception(f"Process has failed!\n {e}")
            except subprocess.TimeoutExpired as e:
                logger.exception(f"Process has expired!\n {e}")
            except Exception as e:
                logger.error(f"Process has failed!\n {e}")
                raise
            else:
                run_end_time = datetime.now()
                logger.info(
                    f'Running --> "{cmd}" has ended on {run_end_time} '
                    f"(took {run_end_time - run_start_time})"
                )
        else:
            logger.error(f"{local_filepath} not found!")

        return process



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from poppy.core.logger import logger
from roc.idb.parsers.idb_elements import Packet, ParameterDefinition, Parameter

__all__ = [
    "PacketParser",
]


class PacketParser:
    """
    Mixin class used to parse packets
    """

    def create_packets_idb(self):
        """
        Create all packets defined in the IDB.
        """
        self.create_packets(self.root)

    def create_packets(self, node):
        """
        Create packets from the packet definitions encountered in the given
        node.
        """
        # prepare the enumeration table

        enumeration_table = dict()

        for enumeration_entry in self._findall(self.root, ".//{0}EnumDefinition", ""):
            # get the enumeration id
            enumeration_id = enumeration_entry.attrib["ID"]

            for value_entry in self._findall(enumeration_entry, ".//{0}Value", ""):
                _ = enumeration_id + "/" + value_entry.attrib["ID"]

                # Check if the ID is already used. If not create a new
                # dictionary to store parent IDs otherwise, add a new key to
                # the dictionary
                if enumeration_table.get(value_entry.attrib["ID"]) is None:
                    enumeration_table[value_entry.attrib["ID"]] = dict(
                        [
                            (
                                enumeration_id,
                                int(value_entry.attrib["NumValue"], 0),
                            )
                        ]
                    )
                else:
                    enumeration_table[value_entry.attrib["ID"]][enumeration_id] = int(
                        value_entry.attrib["NumValue"], 0
                    )

                # NumValue can be both decimal and hex values but with the 0x
                # prefix, Python can distinguish hex and decimal automatically
                # We must specify 0 as the base in order to invoke this
                # prefix-guessing behavior

        # prepare the dict of structured that may be included in packets

        _ = {
            structure_node.attrib["ID"]: structure_node
            for structure_node in self._findall(self.root, ".//{0}StructDefinition", "")
        }

        _ = {
            param_node.attrib["StructParamIDRef"]: param_node.attrib["Value"]
            for param_node in self._findall(
                self.root, ".//{0}Param[@StructParamIDRef]", ""
            )
        }

        # search for packets in the given node
        for packet_node in self._findall(
            node,
            ".//{0}PacketDefinition",
            "",
        ):
            # get the name of the packet
            palisade_id = packet_node.attrib["Name"]

            # search for the SRDB id of the packet
            if palisade_id[:2] not in ["TM", "TC"]:
                logger.warning("Skipping packet %s" % palisade_id)
                continue
            elif palisade_id in self.palisade_to_srdb_id[palisade_id[:2]]:
                # keep reference to mapping
                info = self.packets_mapping[palisade_id]

            else:
                logger.error(
                    (
                        "Packet {0} not found in PALISADE. Maybe an internal DPU"
                        + " command ?"
                    ).format(palisade_id)
                )
                continue

            # get the type of the packet
            packet_type = self._find(
                packet_node,
                ".//{0}Param[@StructParamIDRef='PACKET_TYPE']",
                "",
            )
            if packet_type is None:
                # the packet doesn't have a type, so don't analyze it
                logger.warning(
                    (
                        "Packet {0} doesn't have a type. Surely an internal "
                        + "command of the DPU."
                    ).format(palisade_id)
                )
                continue
            else:
                pack_type = packet_type.attrib["Value"][:2]

            # create the object of the packet
            packet = Packet(
                palisade_id,
                info["SRDBID"],
                packet_node,
                type=pack_type,
            )

            packet.spid = info["SPID"] if "SPID" in info else None

            # get the engineering values for the category, type, subtype, sid
            # and pid

            # get the packet category
            packet.category_eng = self._find(
                packet_node,
                ".//{0}Param[@StructParamIDRef='PACKET_CATEGORY']",
                "",
            ).attrib["Value"]

            # get the packet service_type
            packet.service_type_eng = self._find(
                packet_node,
                ".//{0}Param[@StructParamIDRef='SERVICE_TYPE']",
                "",
            ).attrib["Value"]

            # get the packet service_subtype
            packet.service_subtype_eng = self._find(
                packet_node,
                ".//{0}Param[@StructParamIDRef='SERVICE_SUBTYPE']",
                "",
            ).attrib["Value"]

            # get the packet pid
            pid_node = self._find(
                packet_node, ".//{0}Param[@StructParamIDRef='PROCESS_ID']", ""
            )

            packet.pid_eng = pid_node.attrib["Value"] if pid_node is not None else None

            # now get the raw values using the enumeration mapping table

            # extract the key, value tuple from the mapping table and manage
            # cases where there are 0 or more than 1 element for the given
            # key.

            # Packet category can't be NULL

            ((_, packet.category),) = enumeration_table[packet.category_eng].items()

            if packet.pid_eng is not None:
                ((_, packet.pid),) = enumeration_table[packet.pid_eng].items()
            else:
                packet.pid = None

            if packet.service_type_eng is not None:
                ((_, packet.service_type),) = enumeration_table[
                    packet.service_type_eng
                ].items()
            else:
                packet.service_type = None

            if packet.service_subtype_eng is not None:
                if len(enumeration_table[packet.service_subtype_eng]) == 1:
                    ((_, packet.service_subtype),) = enumeration_table[
                        packet.service_subtype_eng
                    ].items()
                else:
                    packet.service_subtype = enumeration_table[
                        packet.service_subtype_eng
                    ]["SERVICE_SUBTYPE_" + packet.service_type_eng]
            else:
                packet.service_subtype = None

            # get the description of the packet
            packet.description = self._find(packet_node, "{0}Description", "").text

            # get the palisade category
            palisade_category = self.get_palisade_category(packet_node)

            # compute the long srdb id
            srdb_id = self.compute_packet_srdbid(pack_type, info["SRDBID"])

            # store the palisade id and the palisade category indexed by the srdb id
            self.palisade_metadata_map[srdb_id] = {
                "palisade_id": palisade_id,
                "palisade_category": palisade_category,
            }

            # create parameters in relation with the packet
            packet.parameters = self.create_parameters_from_packet(packet)

        return enumeration_table

    def _get_sid_from_structure(self, current_node, parameter):
        param_node = self._find(
            current_node,
            ".//{0}Parameter[@IDRef='%s']" % parameter.definition.palisade_id,
            "",
        )

        if param_node is None:
            param_node = self._find(
                current_node,
                ".//{0}Param[@StructParamIDRef='%s']"
                % parameter.definition.palisade_id,
                "",
            )

        if parameter.alias is not None and param_node is None:
            param_node = self._find(
                current_node,
                ".//{0}Param[@StructParamIDRef='%s']" % parameter.alias,
                "",
            )
        sid_eng = None

        if param_node is None:
            for child in current_node:
                # if the child is a structure
                if child.tag == self.struct_tag:
                    struct_sid = self._get_sid_from_structure(
                        self.structures[child.attrib["IDRef"]], parameter
                    )
                else:
                    struct_sid = self._get_sid_from_structure(child, parameter)
                if struct_sid is not None:
                    sid_eng = struct_sid
        else:
            sid_eng = param_node.attrib.get("ForceValue")
            if sid_eng is None:
                sid_eng = param_node.attrib.get("Value")
        return sid_eng

    def _get_packet_sid(self, packet, parameter):
        """
        Get the packet sid from the sid parameter
        """
        param_node = self._find(
            packet.node,
            ".//{0}Parameter[@IDRef='%s']" % parameter.definition.palisade_id,
            "",
        )

        # if None check for struct tag
        sid_eng = None

        if param_node is None:
            sid_eng = self._get_sid_from_structure(
                self._find(packet.node, ".//{0}Field[@Name='SOURCE_DATA']", ""),
                parameter,
            )

        else:
            sid_eng = param_node.attrib.get("ForceValue")
        try:
            return sid_eng, parameter.definition.data_type.values[sid_eng]
        except KeyError:
            logger.error(
                "SID tag found but no value found for the packet " + repr(packet)
            )
            return None, None

    def create_parameters_from_packet(self, packet):
        """
        Create parameters associated to a packet with all necessary
        information extracted from the IDB and mapping with SRDB.
        """
        # initialize some parameters for tracking of information
        self.bit_cursor = 0

        # get the parameters from the XML file by recursively parsing nodes
        parameters, _, _ = self._parameters_from_node(
            packet.node,
            packet.type,
        )

        sid_parameters = [parameter for parameter in parameters if parameter.is_sid]

        sid_list_len = len(sid_parameters)

        if sid_list_len == 0:
            logger.debug("No SID for the packet %s" % packet)
        elif sid_list_len == 1:
            logger.debug("SID parameter for the packet %s" % packet)
            packet.sid_eng, packet.sid = self._get_packet_sid(packet, sid_parameters[0])
        else:
            raise RuntimeError("Multiple SID detected for the packet %s" % packet)

        # store the information of the packet size in bytes. This necessary
        # because parameters are not sufficient for this, since sometimes
        # packets have a spare at the end of the packet, and verifying the size
        # of the packet with the last parameter is not sufficient
        if self.bit_cursor % 8 != 0:
            # in case the size is not a multiple of a byte, we have a problem
            # and inform the user
            logger.error(
                (
                    "The length of {0} packet is not a byte multiple. "
                    + "Problem in parsing the IDB.".format()
                )
            )
        packet.byte_size = self.bit_cursor // 8

        # return parameters
        return parameters

    def _parameters_from_node(self, node, packet_type):
        """
        Given a node, search recursively for parameters in structure node if
        present and in the node itself.
        """
        # an empty set of parameters
        parameters = list()

        # block size of parameters and spare inside the node
        block_size = 0

        # store the value of the sid and sid_eng if it exists
        packet_sid = None

        # loop over child nodes
        for child in node:
            sid_value = None
            # if the child is a structure
            if child.tag == self.struct_tag:
                # get the structures definition node
                struct_node = self.structures[child.attrib["IDRef"]]

                # get the parameters from this node
                params, bsize, sid_value = self._parameters_from_node(
                    struct_node,
                    packet_type,
                )
                parameters += params
                block_size += bsize

            # if we have a spare, the bit cursor needs to be moved of the value
            # specified in the spare tag
            elif child.tag == self.spare_tag:
                # get the unit
                unit = child.attrib["Size"]
                # in bit units
                if unit == "Bit":
                    bits = int(child.attrib["Count"])
                    self.bit_cursor += bits
                    block_size += bits
                elif unit == "Byte":
                    bits = int(child.attrib["Count"]) * 8
                    self.bit_cursor += bits
                    block_size += bits

            # if the tag is a loop, this is the case of parameters inside a
            # block and we need to loop on the packet to get the wanted
            # information
            elif child.tag == self.loop_tag:
                # get parameters inside the loop tag
                params, bsize, sid_value = self._parameters_from_node(
                    child, packet_type
                )
                block_size += bsize

                # set the group size to the number of parameters
                group_size = len(params)

                # set the counter value if not a parameter name
                count = child.attrib["Count"]
                if not count.startswith("PARAM:"):
                    counter = int(child.attrib["Count"])
                else:
                    counter = 0

                # loop over parameters for setting the group size
                for param in params:
                    param.group_size = group_size
                    param.block_size = bsize // 8  # in bytes
                    param.group_counter = counter

                # add parameters
                parameters += params

            # if the child is a parameter
            elif child.tag == self.param_tag:
                # get the name of the parameter
                palisade_id = child.attrib["IDRef"]
                srdb_id = self.palisade_to_srdb_id[packet_type][palisade_id]

                # get info on the packet from the IDB
                definition = ParameterDefinition.manager[srdb_id]

                # create a parameter for storing information
                parameter = Parameter(definition, packet_type)

                # store the byte position and bit position inside the byte
                parameter.byte_position = self.bit_cursor // 8
                parameter.bit_position = self.bit_cursor % 8

                # increment the byte position with the size of the parameter
                # and store it
                self.bit_cursor += definition.data_type.bit_length
                block_size += definition.data_type.bit_length

                # add the parameter to the set
                parameters.append(parameter)

            # if something else, search in it to see if a parameter is present
            else:
                params, bsize, sid_value = self._parameters_from_node(
                    child,
                    packet_type,
                )
                if child.attrib.get("TAG") == "SID":
                    if len(params) == 1:
                        params[0].is_sid = True
                        params[0].alias = child.attrib.get("OptionalStructParamID")
                    else:
                        raise RuntimeError(
                            "SID parameters are supposed to"
                            + " be alone in the structure tag"
                        )
                parameters += params
                block_size += bsize

            if sid_value is not None:
                packet_sid = sid_value

        # return parameters
        return parameters, block_size, packet_sid



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import pathlib
import subprocess
from io import BytesIO
from zipfile import ZipFile

import requests
from poppy.core.logger import logger
from poppy.core.task import Task

__all__ = ["InstallIdbTask"]

from roc.idb.constants import (
    TIMEOUT,
    IDB_SVN_URL,
    IDB_SVN_USER,
    IDB_SVN_PASSWORD,
    MIB_ARCHIVE_URL_PATTERN,
    MIB_GITLAB_TOKEN,
)


class IdbInstallError(Exception):
    pass


def download_palisade(
    raw_idb_version,
    install_dir,
    user=IDB_SVN_USER,
    password=IDB_SVN_PASSWORD,
    svn_url=IDB_SVN_URL,
    timeout=TIMEOUT,
    force=False,
):
    """
    Download a IDB PALISADE version.

    :param raw_idb_version: string with PALISADE IDB version
    :param install_dir: string with the IDB installation main directory path
    :param user: login user
    :param password: login password
    :param svn_url: SVN host URL
    :param timeout: process timeout in seconds
    :param force: Force installation even if IDB folder is already found locally
    :return: Path of the directory where the IDB has been downloaded
    """
    idb_version = (
        raw_idb_version if raw_idb_version.startswith("V") else "V" + raw_idb_version
    )

    # build the url with the idb version
    url = f"{svn_url}/{idb_version}"

    # create the intermediate-level directories needed to contain the idb
    idb_dir_path = os.path.join(install_dir, idb_version)

    if os.path.exists(idb_dir_path) and not force:
        logger.warning(
            f"IDB [PALISADE-{idb_version}] already found in {idb_dir_path}: cancelling task! "
            f"(Use --force keyword to force installation)"
        )
        raise FileExistsError

    # Create IDB local folder
    os.makedirs(idb_dir_path, exist_ok=True)

    # build the command
    cmd = "svn checkout --no-auth-cache"
    if user:
        cmd += f" --username {user} "
    if password:
        cmd += f" --password {password} "
    cmd += f"{url} {idb_dir_path}"

    _ = subprocess.run(
        cmd, stdout=subprocess.PIPE, shell=True, timeout=timeout, check=True
    )
    return idb_dir_path


def download_mib(
    idb_version,
    install_dir,
    access_token=MIB_GITLAB_TOKEN,
    url=MIB_ARCHIVE_URL_PATTERN,
    force=False,
):
    """
    Download the MIB using the gitlab API

    :param idb_version: The tag, branch reference, or SHA of the MIB
    :param install_dir: The directory where the mib will be unzipped
    :param access_token: The private access token
    :param url: The gitlab API url
    :param force: Force installation even if local files are already found
    :return: Path where the IDB has been downloaded
    """

    # call the gitlab API using the user token
    headers = None
    if access_token:
        headers = {"PRIVATE-TOKEN": access_token}
    response = requests.get(
        url.format(version=idb_version),
        headers=headers,
    )

    # if the response was successful, no Exception will be raised
    response.raise_for_status()

    # create the intermediate-level directories needed to contain the idb
    idb_dir_path = os.path.join(install_dir, idb_version)

    if os.path.exists(idb_dir_path) and not force:
        logger.warning(
            f"IDB [MIB-{idb_version}] already found in {idb_dir_path}: cancelling task! "
            f"(Use --force keyword to force installation)"
        )
        raise FileExistsError

    # Create IDB local folder
    os.makedirs(idb_dir_path, exist_ok=True)

    # extract the zip
    try:
        with ZipFile(BytesIO(response.content)) as zip_file:
            for zip_info in zip_file.infolist():
                # skip the root dir of the zip
                zip_path = pathlib.Path(zip_info.filename)
                if len(zip_path.parts[1:]) == 0:
                    continue

                # the zip lib use trailing slash to determine if it's a dir or a file
                if zip_info.filename[-1] == "/":
                    # it's a directory, add a trailing slash to the pathlib output
                    zip_info.filename = (
                        pathlib.Path(*zip_path.parts[1:]).as_posix() + "/"
                    )
                else:
                    zip_info.filename = pathlib.Path(*zip_path.parts[1:]).as_posix()

                # extract the file
                zip_file.extract(zip_info, idb_dir_path)
    except Exception as e:
        logger.error(f"Installing IDB [MIB-{idb_version}] has failed!")
        logger.debug(e)
        raise IdbInstallError

    return idb_dir_path


@Task.as_task(plugin_name="roc.idb", name="idb_install")
def InstallIdbTask(self):
    if self.pipeline.args.idb_source in ["SRDB", "PALISADE"]:
        logger.debug(
            f'Downloading PALISADE IDB version: "{self.pipeline.args.idb_version}"'
            f' in "{self.pipeline.args.install_dir}"'
        )
        try:
            download_palisade(
                self.pipeline.args.idb_version,
                self.pipeline.args.install_dir,
                user=self.pipeline.args.svn_user[0],
                password=self.pipeline.args.svn_password[0],
            )
        except subprocess.CalledProcessError:
            logger.exception(
                f"Downloading PALISADE IDB {self.pipeline.args.idb_version} has failed!"
            )
            self.pipeline.exit()
        except subprocess.TimeoutExpired:
            logger.exception(
                f"Time to download PALISADE IDB {self.pipeline.args.idb_version} has expired!"
            )
            self.pipeline.exit()
        except FileExistsError:
            pass
        except IdbInstallError:
            pass

    elif self.pipeline.args.idb_source == "MIB":
        logger.debug(
            f'Downloading MIB IDB version: "{self.pipeline.args.idb_version}"'
            f' in "{self.pipeline.args.install_dir}"'
        )
        try:
            download_mib(
                self.pipeline.args.idb_version,
                self.pipeline.args.install_dir,
                access_token=self.pipeline.args.access_token[0],
            )
        except FileExistsError:
            pass
        except IdbInstallError:
            pass
    else:
        raise IdbInstallError(f"Wrong IDB source '{self.pipeline.args.idb_source}'")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import shutil

from poppy.core.logger import logger
from poppy.core.test import CommandTestCase

from roc.idb.models.idb import IdbRelease

from sqlalchemy.exc import NoResultFound

import pytest

from .idb_test_util import reset_db, load_idb


class TestClearReleaseCommand(CommandTestCase):
    """
    Test the clear_release command of the roc.idb plugin.
    """

    # @pytest.mark.skip
    @pytest.mark.parametrize(
        "idb_source,idb_version",
        [("MIB", "20200113")],
    )
    def test_clear_release(self, idb_source, idb_version):
        # Initializing database
        logger.debug("Reset database ...")
        reset_db(self)

        # Installing IDB
        logger.debug(f"Installing IDB [{idb_source}-{idb_version}] ...")
        self.install_dir = load_idb(self, idb_source, idb_version)

        # Command to test
        command_to_test = [
            "pop",
            "idb",
            "clear_release",
            idb_version,
            "-s",
            idb_source,
            "--force",
            "-ll",
            "ERROR",
        ]

        # Run command to test
        self.run_command(command_to_test)

        # Verify expected behaviour
        try:
            _ = (
                self.session.query(IdbRelease)
                .filter(
                    IdbRelease.idb_version == idb_version,
                    IdbRelease.idb_source == idb_source,
                )
                .one()
            )
        except NoResultFound:
            assert True
        else:
            assert False

    def teardown_method(self, method):
        """
        Method called immediately after the test method has been called and the result recorded.

        This is called even if the test method raised an exception.

        :param method: the test method
        :return:
        """

        # rollback the database
        super().teardown_method(method)

        # Remove IDB folder
        shutil.rmtree(self.install_dir)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


from poppy.core.conf import settings
from poppy.core.db.connector import Connector
from poppy.core.logger import logger
from poppy.core.task import Task
from poppy.core.tools.exceptions import MissingArgument
from roc.idb.constants import IDB_SOURCE
from roc.idb.exceptions import IdbUpdateError, SetTrangeIdbReleaseError
from roc.idb.models.idb import IdbRelease
from sqlalchemy.orm.exc import MultipleResultsFound, NoResultFound

__all__ = ["SetTrangeTask"]


class SetTrangeTask(Task):
    plugin_name = "roc.idb"

    name = "idb_set_trange"

    @Connector.if_connected(settings.PIPELINE_DATABASE)
    def run(self):
        """
        Set the validity time range of a given IDB
        """

        # Get input arguments
        self.idb_version = self.pipeline.get("idb_version", default=None, args=True)
        self.idb_source = self.pipeline.get("idb_source", default=IDB_SOURCE, args=True)
        self.validity_min = self.pipeline.get("validity_min", default=None, args=True)
        self.validity_max = self.pipeline.get("validity_max", default=None, args=True)

        if self.idb_version is None:
            raise MissingArgument('Input argument "idb_version" must be defined!')
        elif self.idb_version.upper().startswith("V"):
            self.idb_version = self.idb_version[1:]

        if self.validity_min is None and self.validity_max is None:
            raise MissingArgument(
                'Input arguments "validity_min" and/or "validity_max" must be defined!'
            )

        # First, check that IDB release exists in the database
        idb = self.get_current_idb()

        if self.validity_min is not None:
            # Check that already defined validity_max is not older than input validity_min
            if idb.validity_max is not None and idb.validity_max < self.validity_min:
                raise IdbUpdateError(
                    f"Input validity_min ({self.validity_min}) is older "
                    f"than the validity_max ({idb.validity_max}) found in the database "
                    f"for IDB {self.idb_source} {self.idb_version}, exit task!"
                )

            # Check that validity_min is not already defined
            if idb.validity_min is not None:
                logger.warning(
                    f'Existing validity_min value "{idb.validity_min}" will be replaced'
                )

            idb.validity_min = self.validity_min
            logger.info(
                f'validity_min value set to "{self.validity_min}" for IDB {self.idb_version}'
            )

        if self.validity_max is not None:
            # Check that already defined validity_min is not younger than input validity_max
            if idb.validity_min is not None and idb.validity_min > self.validity_max:
                raise IdbUpdateError(
                    f"Input validity_max ({self.validity_max}) is younger "
                    f"than the validity_min ({idb.validity_min}) found in the database "
                    f"for IDB {self.idb_source} {self.idb_version}, exit task!"
                )

            # Check that validity_max is not already defined
            if idb.validity_max is not None:
                logger.warning(
                    f'Existing validity_max value "{idb.validity_max}" will be replaced'
                )

            idb.validity_max = self.validity_max
            logger.info(
                f'validity_max value set to "{self.validity_max}" for IDB {self.idb_version}'
            )

    def get_current_idb(self):
        # get all the releases stored in the database
        self.all_releases = self.pipeline.db.session.query(IdbRelease)

        # get the release we want to set
        query_filter = {"idb_version": self.idb_version}
        query_filter["idb_source"] = self.idb_source

        try:
            idb = self.all_releases.filter_by(**query_filter).one()
        except NoResultFound:
            raise SetTrangeIdbReleaseError(
                f"IDB version '{self.idb_version}' not found in the " "MAIN-DB"
            )
        except MultipleResultsFound:
            raise SetTrangeIdbReleaseError(
                "Multiple entries found for IDB "
                f"'{self.idb_version}', please specify an IDB source."
            )
        else:
            logger.debug(f"IDB version {self.idb_version} found in the database")

        return idb



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .palisade_parser import PALISADEParser
from .palisade_metadata_parser import PALISADEMetadataParser

__all__ = [
    "PALISADEParser",
    "PALISADEMetadataParser",
]



# -*- coding: utf-8 -*-
import os
import numpy

from .procedure import Procedure


class ParseSequenceMixin(object):
    def parse_sequences(self):
        """
        Parse MIB sequences
        """

        # the CSF table describes the top level of the sequence hierarchy, i.e. it defines the sequence itself

        csf_dtype = numpy.dtype(
            [
                ("SQNAME", "U8"),
                ("DESC", "U24"),
                ("DESC2", "U64"),
                (
                    "IFTT",
                    "U1",
                ),  # Flag identifying whether or not the sequence contains execution
                # time-tagged commands
                (
                    "NFPARS",
                    "U64",
                ),  # Number of formal parameters of this sequence
                ("ELEMS", "U64"),  # Number of elements in this sequence
                (
                    "CRITICAL",
                    "U1",
                ),  # Flag identifying the sequence 'criticality'
                (
                    "PLAN",
                    "U1",
                ),  # Flag identifying the sources which are allowed to reference
                # directly this sequence and use it in a planning file
                (
                    "EXEC",
                    "U1",
                ),  # Flag controlling whether this sequence can be loaded onto
                # SCOS-2000 manual stacks stand-alone
                (
                    "SUBSYS",
                    "U64",
                ),  # Identifier of the subsystem which the sequence belongs to
                ("TIME", "U17"),  # Time when sequence was generated
                (
                    "DOCNAME",
                    "U32",
                ),  # Document name from which it was generated (Procedure name)
                ("ISSUE", "U10"),  # Issue number of document described above
                ("DATE", "U17"),  # Issue date of document described above
                (
                    "DEFSET",
                    "U8",
                ),  # Name of the default parameter value set for the sequence
                (
                    "SUBSCHEDID",
                    "U64",
                ),  # Identifier of the default on-board sub-schedule to be used when
                # loading this sequence as execution time-tagged
            ]
        )

        csf_table = self.from_table(
            os.path.join(self.dir_path, "csf.dat"),
            dtype=csf_dtype,
            regex_dict={"SQNAME": "^(ANEF|AIW[CFV])[0-9]{3}[A-Z]$"},
        )

        # the CSS table contains an entry for each element in the sequence
        # an element can either be a sequence or command, or it can be a formal parameter of the current sequence

        css_dtype = numpy.dtype(
            [
                (
                    "SQNAME",
                    "U8",
                ),  # Name of the sequence which the element belongs to
                (
                    "COMM",
                    "U32",
                ),  # Additional comment to be associated with the sequence element
                ("ENTRY", int),  # Entry number for the sequence element
                (
                    "TYPE",
                    "U1",
                ),  # Flag identifying the element type (C: Command, T: Textual comment)
                (
                    "ELEMID",
                    "U8",
                ),  # Element (identifier) for the current entry within the sequence
                (
                    "NPARS",
                    int,
                ),  # Number of editable parameters this element has
                (
                    "MANDISP",
                    "U1",
                ),  # Flag controlling whether the command requires explicit manual
                # dispatch in a manual stack
                (
                    "RELTYPE",
                    "U1",
                ),  # Flag controlling how the field CSS_RELTIME has to be used to
                # calculate the command release time-tag or the nested sequence
                # release start time
                (
                    "RELTIME",
                    "U8",
                ),  # This field contains the delta release time-tag for the sequence
                # element
                (
                    "EXTIME",
                    "U17",
                ),  # This field contains the delta execution time-tag for the sequence
                # element
                (
                    "PREVREL",
                    "U1",
                ),  # Flag controlling how the field CSS_EXTIME has to be used to
                # calculate the command execution time-tag or the nested sequence
                # execution start time
                (
                    "GROUP",
                    "U1",
                ),  # This field defines the grouping condition for the sequence element
                (
                    "BLOCK",
                    "U1",
                ),  # This field defines the blocking condition for the sequence element
                ("ILSCOPE", "U1"),
                # Flag identifying the type of the interlock associated to this sequence element
                ("ILSTAGE", "U1"),
                # The type of verification stage to which an interlock associated to this
                # sequence element waits on
                ("DYNPTV", "U1"),
                # Flag controlling whether the dynamic PTV checks shall be overridden for this sequence element
                ("STAPTV", "U1"),
                # Flag controlling whether the static (TM based) PTV check shall be overridden for this sequence element
                ("CEV", "U1"),
                # Flag controlling whether the CEV checks shall be disabled for this sequence element
            ]
        )

        css_table = self.from_table(
            os.path.join(self.dir_path, "css.dat"),
            dtype=css_dtype,
            regex_dict={"SQNAME": "^(ANEF|AIW[CFV])[0-9]{3}[A-Z]$"},
        )

        # the SDF table defines the values of the editable element parameters.

        sdf_dtype = numpy.dtype(
            [
                (
                    "SQNAME",
                    "U8",
                ),  # Name of the sequence which the parameter belongs to
                (
                    "ENTRY",
                    int,
                ),  # Entry number of the sequence element to which this parameter belongs
                (
                    "ELEMID",
                    "U8",
                ),  # Name of the sequence element to which this parameter belongs
                ("POS", int),
                # If the sequence element is a command, then this field specifies the bit offset of the parameter in the command
                (
                    "PNAME",
                    "U8",
                ),  # Element parameter name i.e. name of the editable command parameter
                ("FTYPE", "U1"),
                # Flag controlling whether the parameter value can be modified after loading the sequence on the stack or if it should be considered as fixed
                ("VTYPE", "U1"),
                # Flag identifying the source of the element parameter value and how to interpret the SDF_VALUE field
                (
                    "VALUE",
                    "U17",
                ),  # This field contains the value for the element parameter
                ("VALSET", "U8"),
                # Name of the parameter value set (PSV_PVSID) which will be used to provide the value
                ("REPPOS", "int"),  # Command parameter repeated position
            ]
        )

        sdf_table = self.from_table(
            os.path.join(self.dir_path, "sdf.dat"),
            dtype=sdf_dtype,
            regex_dict={"SQNAME": "^(ANEF|AIW[CFV])[0-9]{3}[A-Z]$"},
        )

        # the CSP defines the formal parameters associated with the current sequence

        csp_dtype = numpy.dtype(
            [
                (
                    "SQNAME",
                    "U8",
                ),  # Name of the sequence which formal parameter belongs to
                ("FPNAME", "U8"),  # Name of this formal parameter
                (
                    "FPNUM",
                    int,
                ),  # Unique number of the formal parameter within the sequence
                (
                    "DESCR",
                    "U24",
                ),  # Textual description of the formal parameter
                ("PTC", int),  # Parameter Type Code
                ("PFC", int),  # Parameter Format Code
                ("DISPFMT", "U1"),
                # Flag controlling the input and display format of the engineering values for calibrated parameters and for time parameters
                ("RADIX", "U1"),
                # This is the default radix that the raw representation of the parameter if it is unsigned integer
                (
                    "TYPE",
                    "U1",
                ),  # Flag identifying the type of the formal parameter
                ("VTYPE", "U1"),
                # Flag identifying the representation used to express the formal parameter default value
                (
                    "DEFVAL",
                    "U17",
                ),  # This field contains the default value for this formal parameter
                ("CATEG", "U1"),
                # Flag identifying the type of calibration or special processing associated to the parameter
                ("PRFREF", "U10"),
                # This field contains the name of an existing parameter range set to which the parameter will be associated
                ("CCAREF", "U10"),
                # This field contains the name of an existing numeric calibration curve to which the parameter will be associated
                ("PAFREF", "U10"),
                # This field contains the name of an existing textual calibration definition to which the parameter will be associated
                ("UNIT", "U4"),  # Engineering unit mnemonic
            ]
        )

        csp_table = self.from_table(
            os.path.join(self.dir_path, "csp.dat"),
            dtype=csp_dtype,
        )

        # filter tables to keep only RPW sequences and return tables

        # filter tables to keep only RPW sequences and create the procedure list
        return Procedure.create_from_MIB(
            csf_table,
            css_table,
            sdf_table,
            csp_table,
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from poppy.core.generic.metaclasses import Singleton
from poppy.core.logger import logger

__all__ = ["IDBToCDF"]


class IDBToCDF(object, metaclass=Singleton):
    """
    A class to convert a value to the good type of the CDF variable.
    """

    def __init__(self):
        self.mapping = {
            "b1Bool": "CDF_UINT1",
            "bU1": "CDF_UINT1",
            "bU2": "CDF_UINT1",
            "bU3": "CDF_UINT1",
            "bU4": "CDF_UINT1",
            "bU5": "CDF_UINT1",
            "bU6": "CDF_UINT1",
            "bU7": "CDF_UINT1",
            "bU8": "CDF_UINT1",
            "bU9": "CDF_UINT2",
            "bU10": "CDF_UINT2",
            "bU12": "CDF_UINT2",
            "UShort": "CDF_UINT2",
            "Short": "CDF_INT2",
            "bU24": "CDF_UINT4",
            "UInt": "CDF_UINT4",
            "b10": "CDF_INT2",
            "4": "CDF_UINT4",
            "Float": "CDF_REAL4",
        }

    def __call__(self, value):
        if value not in self.mapping:
            logger.error("{0} type not mapped".format(value))
            return
        return self.mapping[value]


# vim: set tw=79 :



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .cdf_fill import *  # noqa: F403
from .idb_to_bit import *  # noqa: F403
from .idb_to_cdf import *  # noqa: F403
from .idb_to_numpy import *  # noqa: F403



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from roc.idb.parsers.srdb_parser import *  # noqa: F403
from roc.idb.parsers.palisade_parser import *  # noqa: F403
from roc.idb.parsers.mib_parser import *  # noqa: F403



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import json

import numpy
from poppy.core.generic.metaclasses import SingletonManager
from roc.idb.converters import IDBToBit as BitConverter
from roc.idb.converters import IDBToCDF as Converter
from roc.idb.converters import IDBToNumpy as NumpyConverter

__all__ = [
    "Packet",
    "Parameter",
    "ParameterDefinition",
    "Enumeration",
    "drop_numpy_type",
]


def drop_numpy_type(class_instance):
    def numpy_to_builtin_type(value):
        if isinstance(value, numpy.generic):
            return value.item()
        else:
            return value

    for key in class_instance.__dict__:
        setattr(
            class_instance,
            key,
            numpy_to_builtin_type(getattr(class_instance, key)),
        )


class NumpyEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, numpy.integer):
            return int(obj)
        elif isinstance(obj, numpy.floating):
            return float(obj)
        elif isinstance(obj, numpy.ndarray):
            return obj.tolist()
        else:
            return super(NumpyEncoder, self).default(obj)


class Packet(metaclass=SingletonManager):
    """
    Representation of a packet.
    """

    def __init__(self, name, srdbid, node=None, kind="normal", type=None, bit_size=0):
        self.name = name
        # remove leading 0
        self.srdbid = str(int(srdbid))
        self.kind = kind
        self.node = node
        self._type = type
        self.byte_size = bit_size // 8
        self.bit_size = bit_size
        self.sid = None
        self.sid_eng = None
        self.service_type = None
        self.service_subtype = None

    def __repr__(self):
        if self.type == "TM":
            prefix = "YIW"
        else:
            prefix = "ZIW"
        return prefix + "{0}".format(self.srdbid.zfill(5))

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, value):
        self._type = value

    def add_to_size(self, bit_length):
        self.bit_size += bit_length
        self.byte_size = self.bit_size // 8

    def update_size(self, bit_length):
        self.bit_size = bit_length
        self.byte_size = self.bit_size // 8

    def set_group_properties(self, group_size, block_size=None, spare_size=0):
        grp_bit_size = 0
        params_list_len = len(self.parameters)
        if block_size is None:
            for param in self.parameters[params_list_len - group_size :]:
                grp_bit_size += param.definition.data_type.bit_length
        else:
            grp_bit_size = block_size
        for param in self.parameters[params_list_len - group_size :]:
            param.group_size = group_size
            param.block_size = (grp_bit_size + spare_size) // 8

    def to_dict(self):
        packet_dict = {
            "parameters": {
                repr(parameter): parameter.to_dict() for parameter in self.parameters
            }
        }
        packet_dict.update(
            {
                key: getattr(self, key)
                for key in self.__dict__
                if key != "node"
                and key != "parameters"
                and "_eng" not in key
                and key != "bit_size"
                and key != "description"
            }
        )
        return {self.name: packet_dict}

    def dump(self):
        return [
            "Packet.%s = %s" % (key, getattr(self, key))
            for key in sorted(list(self.__dict__))
            if key != "node"
        ]

    def to_json(self):
        return json.dumps(self.to_dict(), sort_keys=True, indent=4, cls=NumpyEncoder)


class Parameter(object):
    """
    Simple class container for the parameter definitions and types.
    """

    def __init__(self, definition, packet_type, unit=None):
        self.definition = definition
        self.type = packet_type
        self.group_size = 0
        self.group_counter = 0
        self.block_size = 0
        # if True this parameter contains the packet sid
        self.is_sid = False
        # alternative id used for some parameters (why ???)
        self.alias = None
        self.unit = unit

    def __lt__(self, other):
        self_value = int(self.name[3:].replace("G", "2").replace("D", "1"))
        self_other = int(other.name[3:].replace("G", "2").replace("D", "1"))
        return self_value < self_other

    @property
    def kind(self):
        if self.srdb_id[3] == "G":
            return "global"
        elif self.srdb_id[3] == "D":
            return "detailed"
        else:
            return "normal"

    @property
    def srdb_id(self):
        return self.definition.srdb_id

    @property
    def name(self):
        return self.definition.srdb_id

    @classmethod
    def nameFromSRDBID(cls, short_srdb_id, packet_type, kind):
        """
        Return the name of the parameter in the SRDB rules.
        """
        if kind == "normal":
            letter = ""
            size = 5
        elif kind == "global":
            letter = "G"
            size = 4
        elif kind == "detailed":
            letter = "D"
            size = 4

        if packet_type == "TM":
            prefix = "NIW"
        else:
            prefix = "PIW"
        return prefix + letter + "{0}".format(short_srdb_id.zfill(size))

    def __repr__(self):
        return self.name

    def dump(self):
        return [
            "Parameter.%s = %s" % (key, getattr(self, key))
            for key in sorted(list(self.__dict__))
        ]

    def to_dict(self):
        parameter_dict = {
            "definition - %s" % repr(self.definition): self.definition.to_dict()
        }
        parameter_dict.update(
            {key: getattr(self, key) for key in self.__dict__ if key != "definition"}
        )
        return parameter_dict

    def is_enumeration(self):
        """
        Return True if a parameter is an enumeration.
        """
        return isinstance(self.definition.data_type, Enumeration)


class ParameterDefinition(metaclass=SingletonManager):
    """
    For storing parameter information from their definition in the IDB.
    """

    def __init__(self, srdb_id, **kwargs):
        self.srdb_id = srdb_id

        for key in kwargs:
            setattr(self, key, kwargs[key])

    @property
    def name(self):
        return self.srdb_id

    def __repr__(self):
        return self.name

    def dump(self):
        return [
            "ParameterDefinition.%s = %s" % (key, getattr(self, key))
            for key in sorted(list(self.__dict__))
            if key != "node"
        ]

    def to_dict(self):
        definition_dict = {
            key: getattr(self, key)
            for key in self.__dict__
            if key != "node" and key != "data_type" and key != "description"
        }
        definition_dict.update({"data_type": repr(self.data_type)})
        return definition_dict


class Enumeration(metaclass=SingletonManager):
    """
    Class for storing information on enumerations from their definition.
    """

    def __init__(self, srdb_id, palisade_id=None):
        self.srdb_id = srdb_id
        self.palisade_id = palisade_id
        self._palisade_type = None
        self._values = dict()
        self._sub_enum_list = []

    def __repr__(self):
        if self._palisade_type is None:
            return f"Enum.None ({self.name})"
        else:
            return f"Enum.{self._palisade_type} ({self.name})"

    @property
    def name(self):
        return self.srdb_id

    @property
    def values(self):
        val = self._values
        for enum in self._sub_enum_list:
            val.update(enum.values)
        return val

    @values.setter
    def values(self, val):
        if isinstance(val, dict):
            self._values = val
        elif isinstance(val, list):
            self._sub_enum_list = val
        else:
            raise TypeError("Unsupported type for enumeration values")

    @property
    def data_type(self):
        return self._data_type

    @data_type.setter
    def data_type(self, value):
        self._palisade_type = value
        self._data_type = Converter()(value)
        self._bit_length = BitConverter()(value)
        self._numpy_type = NumpyConverter()(value)

    @property
    def numpy_type(self):
        return self._numpy_type

    @property
    def bit_length(self):
        return self._bit_length

    def dims(self):
        return 0

    def sizes(self):
        return None

    def record_variance(self):
        return "T"

    def dimension_variances(self):
        return None


class Time(object):
    """
    Representation of time.
    """

    def __init__(self, coding, coarse, fine):
        self.coding = coding
        self.coarse = coarse
        self.fine = fine

    def __repr__(self):
        return "Time.coding:%s/coarse:%s/fine:%s" % (
            self.coding,
            self.coarse,
            self.fine,
        )

    @property
    def data_type(self):
        return Converter()(max(self.coarse, self.fine))

    @property
    def numpy_type(self):
        return "time"

    @property
    def bit_length(self):
        return 48

    def dims(self):
        return 1

    def sizes(self):
        return 3

    def record_variance(self):
        return "T"

    def dimension_variances(self):
        return "T"


class SimpleType(object):
    def __init__(self, data_type):
        self.data_type = data_type

    def __repr__(self):
        return self._palisade_type

    @property
    def data_type(self):
        return self._data_type

    @data_type.setter
    def data_type(self, value):
        self._palisade_type = value
        self._data_type = Converter()(value)
        self._bit_length = BitConverter()(value)
        self._numpy_type = NumpyConverter()(value)

    @property
    def numpy_type(self):
        return self._numpy_type

    @property
    def bit_length(self):
        return self._bit_length

    def dims(self):
        return 0

    def sizes(self):
        return None

    def record_variance(self):
        return "T"

    def dimension_variances(self):
        return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import copy
import os.path as osp
import xml.etree.ElementTree as ET

from poppy.core.logger import logger

from .idb_elements import Time, SimpleType

__all__ = [
    "IDBParser",
]


class XMLParser(object):
    """
    Base class for XML parsers.
    """

    @staticmethod
    def parse_and_get_ns(filename):
        events = "start", "start-ns"
        root = None
        ns = {}
        for event, elem in ET.iterparse(filename, events):
            if event == "start-ns":
                if elem[0] in ns and ns[elem[0]] != elem[1]:
                    # NOTE: It is perfectly valid to have the same prefix refer
                    #     to different URI namespaces in different parts of the
                    #     document. This exception serves as a reminder that this
                    #     solution is not robust.    Use at your own peril.
                    raise KeyError("Duplicate prefix with different URI found.")
                ns[elem[0]] = "{%s}" % elem[1]
            elif event == "start":
                if root is None:
                    root = elem
        return ET.ElementTree(root), ns


class IDBParser(XMLParser):
    """
    Base class for IDB parsers.
    """

    def __init__(self, mapping_file_path):
        """
        Store the xml mapping file path and create the mapping dict.
        """
        # mapping file path
        self.mapping_file_path = mapping_file_path

        # mapping tables to link srdb and palisade IDs
        self.srdb_to_palisade_id = {"TM": dict(), "TC": dict()}
        self.palisade_to_srdb_id = {"TM": dict(), "TC": dict()}

        logger.debug("Create mapping from the xml file ...")
        self.create_mapping()
        logger.debug("Create mapping from the xml file ... Done")

        # initialize the mib version (First row of the vdf.dat table including tabs)
        self.mib_version = None

    def source(self):
        """
        Return the source of the IDB
        """
        return self._source

    def store_mapping_tree(self):
        """
        Get nodes for the mapping.
        """
        # check that the file exist
        if self.mapping_file_path is None or not osp.isfile(self.mapping_file_path):
            logger.error("Mapping {0} doesn't exist".format(self.mapping_file_path))
            raise SystemExit(-1)

        # get the tree and namespace
        self.mapping_tree, self._mapping_ns = self.parse_and_get_ns(
            self.mapping_file_path
        )

        # get the root element
        self.mapping_root = self.mapping_tree.getroot()

    def create_mapping(self):
        """
        From the file of the mapping between SRDB and IDB, create parameters
        and packets ((TM,TC,TF) with the good categories.
        """

        self.store_mapping_tree()

        # -- mapping for packets --

        # init the mapping for packets
        self.packets_mapping = {}

        for packet in self.mapping_root.findall(".//Packets//Map"):
            palisade_id = packet.attrib["PALISADEID"]
            # ignore packets which do not start with TM or TC
            packet_type = palisade_id[:2]
            if packet_type not in ["TM", "TC"]:
                logger.warning("Skipping packet %s" % palisade_id)
                continue

            short_srdb_id = packet.attrib["SRDBID"]
            srdb_id = self.compute_packet_srdbid(packet_type, short_srdb_id)
            # store the correspondence between srdb and palisade
            self.srdb_to_palisade_id[packet_type][srdb_id] = palisade_id

            # store the other information
            self.packets_mapping[palisade_id] = copy.copy(packet.attrib)

        # -- mapping for parameters --
        parameters = self.mapping_root.find(".//Parameters")

        # init the mapping for parameters
        self.parameters_mapping = {}
        self.parameters_mapping["TM"] = {}
        self.parameters_mapping["TC"] = {}

        # loop of TM and TC parameters
        types = ["TM", "TC"]
        for parameter_type in types:
            # get the node
            node = parameters.find(parameter_type)

            # search normal, global and detailed parameters
            for parameter in node.findall("Normal/Map"):
                palisade_id = parameter.attrib["PALISADEID"]
                short_srdb_id = parameter.attrib["SRDBID"]
                srdb_id = self.compute_parameter_srdbid(
                    parameter_type, short_srdb_id, "normal"
                )
                self.srdb_to_palisade_id[parameter_type][srdb_id] = palisade_id

                self.parameters_mapping[parameter_type][palisade_id] = {
                    "SRDBID": short_srdb_id,
                    "kind": "normal",
                }

            for parameter in node.findall("Global/Map"):
                palisade_id = parameter.attrib["PALISADEID"]
                short_srdb_id = parameter.attrib["SRDBID"]
                srdb_id = self.compute_parameter_srdbid(
                    parameter_type, short_srdb_id, "global"
                )
                self.srdb_to_palisade_id[parameter_type][srdb_id] = palisade_id

                self.parameters_mapping[parameter_type][palisade_id] = {
                    "SRDBID": short_srdb_id,
                    "kind": "global",
                }

            for parameter in node.findall("Detailed/Map"):
                palisade_id = parameter.attrib["PALISADEID"]
                short_srdb_id = parameter.attrib["SRDBID"]
                srdb_id = self.compute_parameter_srdbid(
                    parameter_type, short_srdb_id, "detailed"
                )
                self.srdb_to_palisade_id[parameter_type][srdb_id] = palisade_id

                self.parameters_mapping[parameter_type][palisade_id] = {
                    "SRDBID": short_srdb_id,
                    "kind": "detailed",
                }

        # init mapping for enumerations
        self.enumerations_mapping = {}
        self.enumerations_mapping["TM"] = {}
        self.enumerations_mapping["TC"] = {}

        # node for enumerations
        enumerations = self.mapping_root.find(".//Enumerations")

        # loop for TM and TC enumerations
        for enum_type in types:
            # get the node
            node = enumerations.find(enum_type)

            # search normal, global and detailed enumerations
            for enumeration in node.findall("Map"):
                palisade_id = enumeration.attrib["PALISADEID"]
                short_srdb_id = enumeration.attrib["SRDBID"]
                srdb_id = self.compute_enumeration_srdbid(
                    enum_type,
                    short_srdb_id,
                )
                self.enumerations_mapping[enum_type][palisade_id] = short_srdb_id

                self.srdb_to_palisade_id[enum_type][srdb_id] = palisade_id

        # -- mapping for TF --
        logger.debug("ENTER TF MAPPING")
        tfs = self.mapping_root.find(".//Calibrations")

        # init the mapping for parameters
        # self.tf_mapping = {}
        # self.tf_mapping["TM"] = {}
        # self.tf_mapping["TC"] = {}

        # loop of TM and TC parameters
        types = ["TM", "TC"]
        for tf_type in types:
            # get the node
            node = tfs.find(tf_type)

            # build the dictionnary srdb : palisade
            for tf in node.findall("Tabulate/Map"):
                palisade_id = tf.attrib["PALISADEID"]
                short_srdb_id = tf.attrib["SRDBID"]

                srdb_id = self.compute_tf_srdbid(
                    tf_type,
                    short_srdb_id,
                )

                self.srdb_to_palisade_id[tf_type][srdb_id] = palisade_id

        logger.debug("CREATE MAPPING TABLE FINISHED SRDB ID --> PALISADE ID")

        # compute the reverse map

        # loop on TM/TC
        for packet_type in types:
            # loop over each srdb_id
            for srdb_id, palisade_id in self.srdb_to_palisade_id[packet_type].items():
                # check for duplicates palisade_id
                if palisade_id in self.palisade_to_srdb_id[packet_type]:
                    if isinstance(
                        self.palisade_to_srdb_id[packet_type][palisade_id],
                        list,
                    ):
                        self.palisade_to_srdb_id[packet_type][palisade_id].append(
                            srdb_id
                        )
                    else:
                        self.palisade_to_srdb_id[packet_type][palisade_id] = [
                            self.palisade_to_srdb_id[packet_type][palisade_id],
                            srdb_id,
                        ]
                else:
                    self.palisade_to_srdb_id[packet_type][palisade_id] = srdb_id

        # remove separation between TM and TC in the srdb_to_palisade_id dict
        # Note: this is not possible in the reverse dictionary as the palisade
        # ID are not really unique
        self.srdb_to_palisade_id = {
            **self.srdb_to_palisade_id["TM"],
            **self.srdb_to_palisade_id["TC"],
        }

        logger.debug("CREATE MAPPING TABLE FINISHED PALISADE ID --> SRDB ID")

    @property
    def mapping_namespace(self):
        return self._mapping_ns

    @staticmethod
    def compute_apid(process_id, packet_category):
        """
        Compute the APID using the process_id and the packet_category
        APID = |0000000|0000|
            process_id | packet_category
        """

        return (process_id << 4) + packet_category  # 4 bits shift

    @staticmethod
    def split_apid(apid):
        """
        Compute the APID using the process_id and the packet_category
        APID = |0000000|0000|
            process_id | packet_category
        """
        # force the type of the apid
        process_id = int(apid) >> 4  # 4 bits shift
        packet_category = int(apid) & 15  # keep only the last 4 bits
        return process_id, packet_category

    @staticmethod
    def split_parameter_SRDBID(srdbid):
        """
        Return the type, the kind and the short srdbid of the parameter using
        the SRDB rules.
        """
        parameter_type = {"NIW": "TM", "PIW": "TC"}
        kind = {"G": "global", "D": "detailed"}

        if srdbid[3] in ["D", "G"]:
            start_srdbid = 4
        else:
            start_srdbid = 3

        return (
            parameter_type[srdbid[:3]],  # type (TM/TC)
            kind.get(srdbid[3], "normal"),  # kind
            int(srdbid[start_srdbid:]),
        )  # short srdbid

    @staticmethod
    def compute_packet_srdbid(packet_type, short_srdbid):
        """
        Return the srdb id according to the short srdb id and the type
        given in argument.
        """
        if packet_type == "TM":
            prefix = "YIW"
        elif packet_type == "TC":
            prefix = "ZIW"
        else:
            raise TypeError("Unknown packet type %s" % packet_type)
        return prefix + "{0}".format(short_srdbid.zfill(5))

    @staticmethod
    def compute_parameter_srdbid(parameter_type, short_srdbid, kind):
        """
        Return the srdb id according to the short srdb id and the type
        given in argument.
        """
        if kind == "normal":
            letter = ""
            size = 5
        elif kind == "global":
            letter = "G"
            size = 4
        elif kind == "detailed":
            letter = "D"
            size = 4

        if parameter_type == "TM":
            prefix = "NIW"
        elif parameter_type == "TC":
            prefix = "PIW"
        else:
            raise TypeError("Unknown parameter type %s" % parameter_type)
        return prefix + letter + "{0}".format(short_srdbid.zfill(size))

    @staticmethod
    def compute_tf_srdbid(tf_type, short_srdbid):
        """
        Return the srdb id according to the short srdb id and the type
        given in argument.
        """
        if tf_type == "TM":
            prefix = "CIWP"
            suffix = "TM"
            return prefix + "{0}".format(short_srdbid.zfill(4)) + suffix
        elif tf_type == "TC":
            prefix = "CIWP"
            suffix = "TC"
            return prefix + "{0}".format(short_srdbid.zfill(4)) + suffix
        else:
            raise TypeError("Unknown TF type %s" % tf_type)

    @staticmethod
    def compute_enumeration_srdbid(enum_type, short_srdbid):
        """
        Return the srdb id according to the short srdb id and the type
        given in argument.
        """
        return "CIWT" + short_srdbid.zfill(4) + enum_type

    @staticmethod
    def srdb_to_palisade_type(ptc, pfc):
        """
        Mapping between srdb and palisade type
        """

        def not_defined(ptc, pfc):
            raise TypeError("Unsupported srdb type (ptc: %i, pfc: %i)" % (ptc, pfc))

        def unsigned_integer_data_type(ptc, pfc):
            if ptc == 2:
                datatype = "bU%i" % pfc
                if datatype == "bU16":
                    datatype = "UShort"
                return SimpleType(datatype)
            elif ptc == 3:
                if pfc < 12:
                    return SimpleType("bU%i" % (pfc + 4))
                elif pfc == 12:
                    return SimpleType("UShort")
                elif pfc == 14:
                    return SimpleType("UInt")
                elif pfc == 16:
                    return SimpleType("UInt64")

            return not_defined(ptc, pfc)

        def boolean_data_type(ptc, pfc):
            return SimpleType("b1Bool")

        def signed_integer_data_type(ptc, pfc):
            if pfc < 12:
                return SimpleType("b%i" % (pfc + 4))
            elif pfc == 12:
                return SimpleType("Short")
            elif pfc == 14:
                return SimpleType("Int")
            elif pfc == 16:
                return SimpleType("Int64")
            return not_defined(ptc, pfc)

        def float_data_type(ptc, pfc):
            if pfc == 1:
                return SimpleType("Float")
            elif pfc == 2:
                return SimpleType("Double")
            return not_defined(ptc, pfc)

        def bit_string_data_type(ptc, pfc):
            return not_defined(ptc, pfc)

        def octet_string_data_type(ptc, pfc):
            return not_defined(ptc, pfc)

        def ascii_string_data_type(ptc, pfc):
            return not_defined(ptc, pfc)

        def absolute_time_data_type(ptc, pfc):
            if pfc == 17:
                # 4 bytes coarse time
                # 2 bytes fine time
                return Time("CUC", 4, 2)

            return not_defined(ptc, pfc)

        def relative_time_data_type(ptc, pfc):
            return not_defined(ptc, pfc)

        ptc_mapping = {
            1: boolean_data_type,
            2: unsigned_integer_data_type,
            3: unsigned_integer_data_type,
            4: signed_integer_data_type,
            5: float_data_type,
            6: bit_string_data_type,
            7: octet_string_data_type,
            8: ascii_string_data_type,
            9: absolute_time_data_type,
            10: relative_time_data_type,
        }

        return ptc_mapping.get(ptc, not_defined)(ptc, pfc)

    @staticmethod
    def split_packet_SRDBID(srdbid):
        """
        Return the name of the parameter in the SRDB rules.
        """
        packet_type = {"YIW": "TM", "ZIW": "TC"}

        return (
            packet_type[srdbid[:3]],  # type (TM/TC)
            "normal",  # kind
            srdbid[3:],
        )  # short srdbid



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from sqlalchemy.orm.exc import NoResultFound, MultipleResultsFound

from poppy.core.conf import settings
from poppy.core.db.connector import Connector
from poppy.core.task import Task
from poppy.core.logger import logger
from poppy.core.tools.exceptions import MissingArgument

from roc.idb.exceptions import NoIdbFound, MultipleIdbFound
from roc.idb.models.idb import IdbRelease


class ClearIdbReleaseTask(Task):
    plugin_name = "roc.idb"

    name = "idb_clear_release"

    @Connector.if_connected(settings.PIPELINE_DATABASE)
    def run(self):
        # Get input arguments
        idb_version = self.pipeline.get("idb_version", default=None, args=True)
        idb_source = self.pipeline.get("idb_source", default=None, args=True)
        force = self.pipeline.get("force", default=False, args=True)

        if idb_version is None:
            raise MissingArgument('Input argument "idb_version" must be defined!')
        elif idb_version.upper().startswith("V"):
            idb_version = idb_version[1:]

        # Get database session
        session = self.pipeline.db.session

        # get all the releases stored in the database
        all_releases = session.query(IdbRelease)

        # get the release we want to set to current
        query_filter = {"idb_version": idb_version}

        if idb_source is not None:
            query_filter["idb_source"] = idb_source

        try:
            idb_to_clear = all_releases.filter_by(**query_filter).one()
        except NoResultFound:
            raise NoIdbFound(
                f"The IDB version '{idb_version}' does not exist in the " "MAIN-DB"
            )
        except MultipleResultsFound:
            raise MultipleIdbFound(
                "Multiple releases exist for the IDB version "
                f"'{idb_version}', please specify an IDB source."
            )
        else:
            if not force:
                answer = input(
                    f"ENTER IDB VERSION TO CONFIRM THE DELETION [{idb_version}]: "
                )
            else:
                answer = idb_version

            if answer == idb_version:
                logger.debug(f"Deleting IDB {idb_version} from database...")
                session.delete(idb_to_clear)
                session.commit()
                logger.info(f"IDB {idb_version} has been removed from the database")
            else:
                logger.info(f"IDB {idb_version} has not been removed from the database")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from poppy.core.command import Command
from roc.idb.tasks import (
    LoadTask,
    LoadSequencesTask,
    LoadPalisadeMetadataTask,
    ParseSequencesTask,
)

__all__ = ["Load", "LoadIdb", "LoadSequences"]


class Load(Command):
    """
    Command to set a given idb version as current
    """

    __command__ = "idb_load"
    __command_name__ = "load"
    __parent__ = "idb"
    __parent_arguments__ = ["base"]
    __help__ = "Load the IDB from files"


class LoadIdb(Command):
    """
    Command to load the information of the IDB into the ROC database.
    """

    __command__ = "idb_load_idb"
    __command_name__ = "idb"
    __parent__ = "idb_load"
    __parent_arguments__ = ["base"]
    __help__ = "Load the IDB packets, parameters and transfer functions from files"

    def add_arguments(self, parser):
        # path to XML file of the IDB
        parser.add_argument(
            "-i",
            "--idb-path",
            help="""
            Path to the IDB directory (e.g. 'V4.3.3/IDB/').
            """,
            type=str,
            required=True,
        )

        # idb source
        parser.add_argument(
            "-t",
            "--idb-source",
            help="""
            IDB type: SRDB or PALISADE or MIB (default: %(default)s).
            """,
            type=str,
            default="MIB",
            required=True,
        )
        # path to the mapping of parameters and packets to the SRDB
        parser.add_argument(
            "-m",
            "--mapping",
            help="""
            Path to the XML file containing the mapping of parameters and
            packets to the SRDB.
            """,
            type=str,
            required=True,
        )

    def setup_tasks(self, pipeline):
        """
        Load the information from the IDB in the given XML files and set them
        into the ROC database.
        """

        load_task = LoadTask()

        # set the pipeline for this situation
        pipeline | load_task
        pipeline.start = load_task


class LoadPalisadeMetadata(Command):
    """
    Command to load the PALISADE metadata into the ROC database.
    """

    __command__ = "idb_load_palisade_metadata"
    __command_name__ = "palisade_metadata"
    __parent__ = "idb_load"
    __parent_arguments__ = ["base"]
    __help__ = "Load the PALISADE metadata from XML files"

    def add_arguments(self, parser):
        # path to XML file of the IDB
        parser.add_argument(
            "idb_path",
            help="""
            Path to the PALISADE IDB directory (e.g. 'V4.3.3/IDB/').
            """,
            type=str,
        )

    def setup_tasks(self, pipeline):
        """
        Load the information from the IDB in the given XML files and set them
        into the ROC database.
        """

        load_metadata_task = LoadPalisadeMetadataTask()

        # set the pipeline for this situation
        pipeline | load_metadata_task
        pipeline.start = load_metadata_task


class LoadSequences(Command):
    """
    Command to load MIB sequences into the ROC database.
    """

    __command__ = "idb_load_sequences"
    __command_name__ = "sequences"
    __parent__ = "idb_load"
    __parent_arguments__ = ["base"]
    __help__ = "Command to load MIB sequences into the MUSIC database"

    def add_arguments(self, parser):
        # path to MIB files directory
        parser.add_argument(
            "mib_dir_path",
            help="""
            Path to MIB files directory.
            """,
            type=str,
        )

    def setup_tasks(self, pipeline):
        """
        Load MIB sequences into the MUSIC database
        """
        # instantiate the task
        parse_sequences = ParseSequencesTask()
        load_sequences = LoadSequencesTask()

        # set the pipeline topology
        pipeline | (parse_sequences | load_sequences)
        pipeline.start = parse_sequences



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from sqlalchemy.orm.exc import NoResultFound, MultipleResultsFound

from poppy.core.conf import settings
from poppy.core.db.connector import Connector
from poppy.core.task import Task
from poppy.core.logger import logger
from poppy.core.tools.exceptions import MissingArgument

from roc.idb.exceptions import NoIdbFound, MultipleIdbFound
from roc.idb.models.idb import IdbRelease
from roc.idb.constants import IDB_SOURCE


class SetCurrentTask(Task):
    plugin_name = "roc.idb"

    name = "idb_set_current"

    @Connector.if_connected(settings.PIPELINE_DATABASE)
    def run(self):
        """
        Set the given IDB version as current
        """
        self.idb_version = self.pipeline.get("idb_version", default=None, args=True)
        self.idb_source = self.pipeline.get("idb_source", default=IDB_SOURCE, args=True)

        if self.idb_version is None:
            raise MissingArgument('Input argument "idb_version" must be defined!')
        elif self.idb_version.upper().startswith("V"):
            self.idb_version = self.idb_version[1:]

        # get all the releases stored in the database
        all_releases = self.pipeline.db.session.query(IdbRelease)

        # get the release we want to set to current
        query_filter = {}
        query_filter["idb_version"] = self.idb_version

        if self.pipeline.args.idb_source:
            query_filter["idb_source"] = self.idb_source

        try:
            future_current_release = all_releases.filter_by(**query_filter).one()
        except NoResultFound:
            raise NoIdbFound(
                f"The IDB version '{self.pipeline.args.idb_version}' does not exist in the "
                "MAIN-DB"
            )
        except MultipleResultsFound:
            raise MultipleIdbFound(
                "Multiple releases exist for the IDB version "
                f"'{self.pipeline.args.idb_version}', please specify an IDB source."
            )

        # set the current field of all releases to False
        all_releases.update({IdbRelease.current: False})

        # set the new current release
        future_current_release.current = True

    @staticmethod
    @Connector.if_connected(settings.PIPELINE_DATABASE)
    def get_latest_idb(idb_source="MIB"):
        """
        Static method to retrieve latest IDB version for a given source

        :param idb_source: IDB source ("MIB" by default)
        :return: entries for latest IDB version (if any)
        """

        # get all the IDB releases stored in the database
        connector = Connector.manager[settings.PIPELINE_DATABASE]
        database = connector.get_database()

        # check the database is connected
        database.connectDatabase()

        # get a database session
        session = database.session_factory()

        all_releases = session.query(IdbRelease)

        # get the latest release we want to get for given idb_source
        query_filter = {"idb_source": idb_source}

        try:
            latest_release = (
                all_releases.filter_by(**query_filter)
                .order_by(IdbRelease.validity_max.desc())
                .all()
            )[0]
        except NoResultFound:
            logger.warning(f"No release found for IDB source '{idb_source}'")
            return []
        else:
            return latest_release



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from roc.idb.tasks.export_sequences import *  # noqa: F403
from roc.idb.tasks.parse_sequences import *  # noqa: F403
from roc.idb.tasks.install import *  # noqa: F403
from roc.idb.tasks.list import *  # noqa: F403
from roc.idb.tasks.load import *  # noqa: F403
from roc.idb.tasks.load_idb_dump import *  # noqa: F403
from roc.idb.tasks.load_sequences import *  # noqa: F403
from roc.idb.tasks.load_palisade_metadata import *  # noqa: F403
from roc.idb.tasks.set_current import *  # noqa: F403
from roc.idb.tasks.set_trange import *  # noqa: F403
from roc.idb.tasks.clear_release import *  # noqa: F403



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Constant variables for roc.idb plugin.
"""

import os
from datetime import datetime

__all__ = [
    "TIME_ARG_STRFORMAT",
    "TIME_SQL_STRFORMAT",
    "VALIDITY_MAX",
    "VALIDITY_MIN",
    "IDB_SOURCE",
    "IDB_INSTALL_DIR",
    "IDB_CACHE_DIR",
    "MIB_VERSION_STRFORMAT",
    "MIB_RELEASE_URL_PATTERN",
    "MIB_ARCHIVE_URL_PATTERN",
    "MIB_GITLAB_TOKEN",
    "IDB_SVN_URL",
    "IDB_SVN_USER",
    "IDB_SVN_PASSWORD",
    "TIMEOUT",
    "POSTGRES_DB_PORT",
    "TIME_WAIT_SEC",
    "TRYOUTS",
    "TIMEOUT",
]

# STRING format for time
TIME_ARG_STRFORMAT = "%Y-%m-%dT%H:%M:%S"
TIME_SQL_STRFORMAT = "%Y-%m-%d %H:%M:%S"

# Default values
VALIDITY_MIN = datetime(2000, 1, 1)
VALIDITY_MAX = datetime(2050, 12, 31)

# Expected input format for MIB version
MIB_VERSION_STRFORMAT = "%Y%m%d"

# Default IDB source
IDB_SOURCE = "MIB"

# Default IDB installation directory path
IDB_INSTALL_DIR = os.environ.get("IDB_INSTALL_DIR", "/pipeline/lib/idb")

# Cache directory path
IDB_CACHE_DIR = "/cache/idb"

# Default URL pattern for the ROC MIB release page
MIB_RELEASE_URL_PATTERN = "https://gitlab.obspm.fr/api/v4/projects/930/packages/generic/mib_release/{mib_version}/idb_dump.sql"

# Default URL pattern for the RPW MIB archive
MIB_ARCHIVE_URL_PATTERN = "https://gitlab.obspm.fr/api/v4/projects/ROC%2Fmib/repository/archive.zip?sha={version}"

# IDB SVN URL
IDB_SVN_URL = "https://version-lesia.obspm.fr/repos/SO-RPW/IDB/tags"
IDB_SVN_USER = os.environ.get("IDB_SVN_USER", None)
IDB_SVN_PASSWORD = os.environ.get("IDB_SVN_PASSWORD", None)

# Gitlab access token
MIB_GITLAB_TOKEN = os.environ.get("MIB_GITLAB_TOKEN", None)

# Default process timeout in seconds
TIMEOUT = 3600

# Postgres database default port
POSTGRES_DB_PORT = 5432

# Database tries and seconds to wait between two tries
TRYOUTS = 3
TIME_WAIT_SEC = 3



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from poppy.core.generic.metaclasses import Singleton
from poppy.core.logger import logger

__all__ = ["CDFToFill", "fill_records", "fill_dict"]


def fill_records(records):
    _filler(records, records.dtype.names)


def fill_dict(records):
    _filler(records, records.keys())


def _filler(records, names):
    # the converter
    converter = CDFToFill()

    # loop over fields
    for name in names:
        # dtype of the record
        dtype = records[name].dtype.name

        # the to fill with
        value = converter(dtype)

        logger.debug(
            "Fill {0} record with {1} for type {2}".format(
                name,
                value,
                dtype,
            )
        )

        # fill with value
        records[name].fill(value)


class CDFToFill(object, metaclass=Singleton):
    """
    To create a mapping between the CDF units and their fill values.
    """

    def __init__(self):
        self.mapping = {
            "CDF_BYTE": -128,
            "CDF_INT1": -128,
            "int8": -128,
            "CDF_UINT1": 255,
            "uint8": 255,
            "CDF_INT2": -32768,
            "int16": -32768,
            "CDF_UINT2": 65535,
            "uint16": 65535,
            "CDF_INT4": -2147483648,
            "int32": -2147483648,
            "CDF_UINT4": 4294967295,
            "uint32": 4294967295,
            "CDF_INT8": -9223372036854775808,
            "int64": -9223372036854775808,
            "uint64": 18446744073709551615,
            "float": -1e31,
            "float16": -1e31,
            "CDF_REAL4": -1e31,
            "CDF_FLOAT": -1e31,
            "float32": -1e31,
            "CDF_REAL8": -1e31,
            "CDF_DOUBLE": -1e31,
            "float64": -1e31,
            "CDF_EPOCH": 0.0,
            "CDF_TIME_TT2000": -9223372036854775808,
            "CDF_CHAR": " ",
            "CDF_UCHAR": " ",
        }

        self.validmin_mapping = {
            "CDF_BYTE": -127,
            "CDF_INT1": -127,
            "CDF_UINT1": 0,
            "CDF_INT2": -32767,
            "CDF_UINT2": 0,
            "CDF_INT4": -2147483647,
            "CDF_UINT4": 0,
            "CDF_INT8": -9223372036854775807,
            "CDF_REAL4": -1e30,
            "CDF_FLOAT": -1e30,
            "CDF_REAL8": -1e30,
            "CDF_DOUBLE": -1e30,
            "CDF_TIME_TT2000": -9223372036854775807,
        }

        self.validmax_mapping = {
            "CDF_BYTE": 127,
            "CDF_INT1": 127,
            "CDF_UINT1": 254,
            "CDF_INT2": 32767,
            "CDF_UINT2": 65534,
            "CDF_INT4": 2147483647,
            "CDF_UINT4": 4294967294,
            "CDF_INT8": 9223372036854775807,
            "CDF_REAL4": 3.4028235e38,
            "CDF_FLOAT": 3.4028235e38,
            "CDF_REAL8": 1.7976931348623157e308,
            "CDF_DOUBLE": 1.7976931348623157e308,
            "CDF_TIME_TT2000": 9223372036854775807,
        }

    def __call__(self, value):
        if value not in self.mapping:
            logger.error("{0} fill type not mapped".format(value))
            return
        return self.mapping[value]

    def validmin(self, value):
        if value not in self.validmin_mapping:
            logger.error("{0} validmin type not mapped".format(value))
            return
        return self.validmin_mapping[value]

    def validmax(self, value):
        if value not in self.validmax_mapping:
            logger.error("{0} validmax type not mapped".format(value))
            return
        return self.validmax_mapping[value]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import csv
import glob
import os

from poppy.core.conf import settings
from poppy.core.db.connector import Connector
from poppy.core.target import FileTarget, PyObjectTarget
from poppy.core.task import Task
from roc.idb.parsers import MIBParser

__all__ = [
    "ParseSequencesTask",
]


def parse_tc_duration_table(file_target):
    # open the duration table and load the data
    with file_target.open() as csv_file:
        csv_reader = csv.reader(csv_file, delimiter=";")

        # skip the headers
        for _ in range(6):
            next(csv_reader)

        table = {}

        for row in csv_reader:
            # get the srdb id of the TC
            srdb_id, palisade_id, duration, comment = row

            table[srdb_id] = {
                "palisade_id": palisade_id,
                "duration": int(duration),
                "comment": comment,
            }

        return table


def parse_sequence_description_table(file_target):
    # open the sequence description table and load the data
    with file_target.open() as csv_file:
        csv_reader = csv.reader(csv_file, delimiter=";")

        # skip the headers
        for _ in range(1):
            next(csv_reader)

        table = {}

        for row in csv_reader:
            # get the srdb id of the TC
            sequence_name, long_description, short_description = row

            table[sequence_name] = {
                "long_description": long_description,
                "description": short_description,
            }

        return table


def load_rpw_states(rpw_state_description_directory_path):
    state_description = {}

    for json_filepath in glob.glob(
        os.path.join(rpw_state_description_directory_path, "*.json")
    ):
        key = os.path.splitext(os.path.basename(json_filepath))[0]
        with open(json_filepath) as json_file:
            state_description[key] = json_file.read()

    return state_description


class ParseSequencesTask(Task):
    plugin_name = "roc.idb"
    name = "idb_parse_sequences"

    def get_tc_duration_table_filepath(self, pipeline):
        return os.path.join(pipeline.args.mib_dir_path, "tc_duration.csv")

    def get_sequence_description_table_filepath(self, pipeline):
        return os.path.join(
            pipeline.args.mib_dir_path, "sequence_description_table.csv"
        )

    def add_targets(self):
        self.add_input(
            target_class=FileTarget,
            identifier="tc_duration_table_filepath",
            filepath=self.get_tc_duration_table_filepath,
        )

        self.add_input(
            target_class=FileTarget,
            identifier="sequence_description_table_filepath",
            filepath=self.get_sequence_description_table_filepath,
        )

        self.add_output(target_class=PyObjectTarget, identifier="sequences")

        self.add_output(target_class=PyObjectTarget, identifier="tc_duration_table")

        self.add_output(
            target_class=PyObjectTarget,
            identifier="sequence_description_table",
        )

        self.add_output(target_class=PyObjectTarget, identifier="rpw_states")

        self.add_output(target_class=PyObjectTarget, identifier="idb_version")

    @Connector.if_connected(settings.PIPELINE_DATABASE)
    def run(self):
        """
        Load MIB sequences into the ROC database
        """

        # get the database session
        self.session = self.pipeline.db.session

        # instantiate the MIB parser

        self.parser = MIBParser(
            os.path.join(self.pipeline.args.mib_dir_path, "data"),
            None,  # no mapping file needed to load sequences
        )

        # parse the sequences
        sequences_data = self.parser.parse_sequences()

        # read the TC duration table
        self.outputs["tc_duration_table"].value = parse_tc_duration_table(
            self.inputs["tc_duration_table_filepath"]
        )

        # read the sequence duration table
        self.outputs[
            "sequence_description_table"
        ].value = parse_sequence_description_table(
            self.inputs["sequence_description_table_filepath"]
        )

        self.outputs["rpw_states"].value = load_rpw_states(
            os.path.join(self.pipeline.args.mib_dir_path, "rpw_states")
        )

        self.outputs["sequences"].value = sequences_data
        self.outputs["idb_version"].value = self.parser.version()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import json
from pprint import pformat

from poppy.core.conf import settings
from poppy.core.db.connector import Connector
from poppy.core.logger import logger
from poppy.core.task import Task
from roc.idb.models.idb import IdbRelease
from roc.idb.models.palisade_metadata import PalisadeMetadata

__all__ = ["ListReleasesTask", "ListPalisadeMetadataTask"]


class ListReleasesTask(Task):
    plugin_name = "roc.idb"

    name = "idb_list_releases"

    @Connector.if_connected(settings.PIPELINE_DATABASE)
    def run(self):
        """
        List the different idb releases
        """
        idb_release_query = self.pipeline.db.session.query(IdbRelease).all()

        idb_count = len(idb_release_query)
        if idb_count > 0:
            logger.info("IDB release available:")
            for idb_release in idb_release_query:
                logger.info(
                    json.dumps(
                        {
                            "source": idb_release.idb_source,
                            "version": idb_release.idb_version,
                            "validity_min": str(idb_release.validity_min),
                            "validity_max": str(idb_release.validity_max),
                            "current": idb_release.current,
                        }
                    )
                )
        else:
            logger.info("No IDB release stored in the database")


class ListPalisadeMetadataTask(Task):
    plugin_name = "roc.idb"

    name = "idb_list_palisade_metadata"

    @Connector.if_connected(settings.PIPELINE_DATABASE)
    def run(self):
        """
        List the palisade metadata
        """

        palisade_metadata_query = self.pipeline.db.session.query(PalisadeMetadata).all()

        logger.info("Palisade metadata list:")

        fields = [
            "palisade_version",
            "srdb_id",
            "palisade_id",
            "packet_category",
        ]

        data = [
            {field: getattr(palisade_metadata, field) for field in fields}
            for palisade_metadata in palisade_metadata_query
        ]

        logger.info(pformat(data))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from roc.idb.parsers.idb_elements import (
    ParameterDefinition,
    Parameter,
    Time,
    SimpleType,
    Enumeration,
)

__all__ = ["ParameterParser"]


class ParameterParser:
    """
    Mixin class used to parse parameters
    """

    def populate_parameters(self):
        """
        Get information on parameters from the IDB, and store them to be
        easily accessible later.
        """
        # look for the nodes in the IDB
        nodes = self._findall(self.root, ".//{0}ParameterDefinition", "")

        # loop over nodes
        for node in nodes:
            # name of the parameter
            palisade_id = node.attrib["ID"]

            # store the palisade id of the associated tf/enum if any
            tf_palisade_id = None

            # initialize the dict containing the parameter definition info
            definition_dict = {
                "palisade_id": palisade_id,
                "maximum": None,
                "minimum": None,
                "description": None,
                "default": None,
                "unit": None,
                "transfer_function": None,
                "is_editable": None,
            }

            # give the description
            definition_dict["description"] = self._find(node, "{0}Description", "").text

            # store also the node of the definition
            definition_dict["node"] = node

            # if the paramater has a max, min, unit or default value, store it
            if "Max" in node.attrib:
                definition_dict["maximum"] = node.attrib["Max"]
            if "Min" in node.attrib:
                definition_dict["minimum"] = node.attrib["Min"]
            if "Unit" in node.attrib:
                definition_dict["unit"] = node.attrib["Unit"]
            if "Default" in node.attrib:
                definition_dict["default"] = str(node.attrib["Default"])

            if "TFIDRef" in node.attrib:
                tf_palisade_id = node.attrib["TFIDRef"]

            # look for fixed parameters
            if (definition_dict["minimum"] is None) or (
                definition_dict["maximum"] is None
            ):
                definition_dict["is_editable"] = None
            else:
                definition_dict["is_editable"] = not (
                    definition_dict["minimum"] == definition_dict["maximum"]
                )

            # get the parameter type from one of the following nodes

            enum = self._find(node, "{0}Enum", "")
            simple_type = self._find(node, "{0}SimpleType", "")
            time = self._find(node, "{0}Time", "")

            is_enum = False

            # if a simple type
            if simple_type is not None:
                definition_dict["data_type"] = SimpleType(simple_type.attrib["Type"])

            # check if enum
            elif enum is not None:
                is_enum = True

                # get the reference of the enum as the transfer function id
                tf_palisade_id = enum.attrib["IDRef"]

            # check if time
            elif time is not None:
                definition_dict["data_type"] = Time(
                    time.attrib["Coding"],
                    self._find(time, "{0}Coarse", "").attrib["Len"],
                    self._find(time, "{0}Fine", "").attrib["Len"],
                )

            # get the corresponding srdb_id(s)
            if self.parameters_mapping is None:
                raise Exception(
                    "The mapping table is required to load the parameter definitions"
                )
            else:
                srdb_id_maps = {
                    packet_type: self.parameters_mapping[packet_type].get(
                        palisade_id, None
                    )
                    for packet_type in ["TM", "TC"]
                }
                srdb_id_list = [
                    (
                        packet_type,
                        Parameter.nameFromSRDBID(
                            srdb_id_map["SRDBID"],
                            packet_type,
                            srdb_id_map["kind"],
                        ),
                    )
                    for packet_type, srdb_id_map in srdb_id_maps.items()
                    if srdb_id_map is not None
                ]

            for packet_type, srdb_id in srdb_id_list:
                # create the parameter from the srdb_id and the info dict

                # check for enum
                if is_enum:
                    # create the enumeration and assign it as data_type
                    enum_srdb_id = self.palisade_to_srdb_id[packet_type][tf_palisade_id]
                    definition_dict["data_type"] = Enumeration.manager[enum_srdb_id]
                    definition_dict["transfer_function"] = enum_srdb_id
                elif tf_palisade_id is not None:
                    # get the srdb_id of the tf
                    tf_srdb_id = self.palisade_to_srdb_id[packet_type].get(
                        tf_palisade_id, None
                    )
                    definition_dict["transfer_function"] = tf_srdb_id

                ParameterDefinition(srdb_id, **definition_dict)
